/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.tasks;

import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Injector;
import org.eclipse.gmt.am3.tools.ant.toolkit.AM3AntToolKit;
import org.eclipse.gmt.am3.tools.ant.toolkit.AM3BuildListener;
import org.eclipse.m2m.atl.engine.AtlModelHandler;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModel;

/**
 * 
 * @author Freddy Allilaire
 *
 */
public class LoadModelTask extends Task {
	
	protected String name;
	protected String metamodel;
	protected String modelHandler;
	
	protected String path;
	protected String nsUri;
	
	private List injectors = new ArrayList();
	
	public void execute() throws BuildException {
		AM3AntToolKit.attachLogHandler(getProject());
		
		log("Loading of model " + name);
//		System.out.println("Loading of model " + name);
		if (name == null)
			throw new BuildException("name attribute must not be null");
		if (metamodel == null)
			throw new BuildException("metamodel attribute must not be null");
		// TODO XOR
		if (path == null && nsUri == null)
			throw new BuildException("path attribute must not be null");
		
		if (modelHandler == null)
			modelHandler = AtlModelHandler.AMH_EMF;
		
//		final AtlModelHandler amh = AtlModelHandler.getDefault(modelHandler);
//		final ModelLoader ml = AM3AntToolKit.getModelLoader(getProject(), amh);
//		final String amhRef = "%" + modelHandler;
//		if (getProject().getReference(amhRef) == null) {
//			getProject().addReference(amhRef, ml.getMOF());
			//Metametamodels should never be disposed
			//getProject().addBuildListener(new AM3BuildListener(amhRef));
//		}
		
		ASMModel loadedModel;
		
		if (getProject().getReference(name) == null) {
		
			if (injectors.isEmpty())
				loadedModel = AM3AntToolKit.loadModel(name, modelHandler, metamodel, path, nsUri, getProject());
			else if (injectors.size() > 1)
				throw new BuildException("Only one injector type can be use at the same time.");
			else {
				// TODO nsUri non g�r�
				Injector injector = (Injector)injectors.get(0);
				loadedModel = AM3AntToolKit.inject(name, path, metamodel, modelHandler, injector, getProject());
			}
			getProject().addReference(name, loadedModel);
			getProject().addBuildListener(new AM3BuildListener(name));
		} else {
			throw new BuildException("Model named " + name + " has already been loaded.");
		}
	}

	public void addConfiguredInjector(Injector i) {
		injectors.add(i);
	}
	
	public void setMetamodel(String metamodel) {
		this.metamodel = metamodel;
	}

	public void setModelHandler(String modelHandler) {
		this.modelHandler = modelHandler;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public void setNsUri(String nsUri) {
		this.nsUri = nsUri;
	}
	
}
