/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.tasks;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.InModel;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Library;
import org.eclipse.gmt.am3.tools.ant.toolkit.AM3AntToolKit;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMBoolean;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMOclAny;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMString;
import org.eclipse.m2m.atl.ocl.core.OclQuery;

/**
 * 
 * @author Freddy Allilaire
 *
 */
public class QueryTask extends Task {

	// Exactly one of body and path must be specified
	// Exactly one of property and file must be specified

	// A query to execute
	protected String body;
	// Path to an asm or atl file containing an ATL!Query
	protected String path;
	
	// The name of a property to set the result of the query to
	protected String property;

	// A file to store the result of the query to
	protected String file;
	
	protected List inModels = new ArrayList();
	
	protected List librarys = new ArrayList();

	public void execute() throws BuildException {
		AM3AntToolKit.attachLogHandler(getProject());
		
		log("Executing query");
//		System.out.println("Executing query");
		
		if (body == null)
			throw new BuildException("body attribute must not be null");
		if (property == null)
			throw new BuildException("property attribute must not be null");
		
		ASMOclAny cls;
		try {
			Map models = new HashMap();
			for (Iterator it = inModels.iterator(); it.hasNext();) {
				InModel in = (InModel)it.next();
				models.put(in.getName(), getProject().getReference(in.getModel()));
			}
			Map libs = new HashMap();
			for (Iterator it = librarys.iterator(); it.hasNext();) {
				Library l = (Library)it.next();
				libs.put(l.getName(), AM3AntToolKit.getURLFromASMFile(l.getPath(), getProject().getBaseDir().getAbsolutePath()));
			}
			cls = new OclQuery(body).eval(models, libs);
			
			if (cls instanceof ASMBoolean) {
				if(((ASMBoolean)cls).getSymbol())
					getProject().setProperty(property, cls.toString());
			} else if (cls instanceof ASMString) {
				getProject().setProperty(property, ((ASMString)cls).getSymbol());				
			} else {
				getProject().setProperty(property, cls.toString());								
			}
		} catch (Exception e) {
			log(e.getLocalizedMessage());
//			e.printStackTrace();
		}
	}
	
	public void addConfiguredInModel(InModel inModel) {
		inModels.add(inModel);
	}
	
	public void setBody(String body) {
		this.body = body;
	}

	public void setProperty(String property) {
		this.property = property;
	}
	public void addConfiguredLibrary(Library library) {
		librarys.add(library);
	}

}
