/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.tasks;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Extractor;
import org.eclipse.gmt.am3.tools.ant.toolkit.AM3AntToolKit;
import org.eclipse.m2m.atl.engine.vm.ModelLoader;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModel;

/**
 * 
 * @author Freddy Allilaire
 *
 */
public class SaveModelTask extends Task {

	protected String model;
	protected String path;
	
	private List extractors = new ArrayList();
	
	public void execute() throws BuildException {
		AM3AntToolKit.attachLogHandler(getProject());
		
		log("Saving model " + model);
		
		ASMModel m = (ASMModel)getProject().getReference(model);
		if(m == null)
			throw new BuildException("model " + model + " does not exist");
		final ModelLoader ml = m.getModelLoader();
		
		if (extractors.isEmpty()) {
			// TODO retrieve modelHandler of the model
			
			if (AM3AntToolKit.isAFileSystemPath(path, getProject(), true))
				path = AM3AntToolKit.convertFileSystemPathToResourcePath(path, getProject(), true);

			if (!AM3AntToolKit.isAResourcePath(path, getProject(), true))
				throw new BuildException("The path is incorrect.");
			
			try {
				ml.save((ASMModel)getProject().getReference(model), path);
			} catch (IOException e) {
				throw new BuildException(e);
			}
		}
		else {
			Extractor extractor = (Extractor)extractors.get(0);
//			<am3.saveModel model="atom_model" path="/News2RSS&amp;HTML/Model/generatedFromNews-ATOM.ecore" >
//				<extractor path="xmi">
//					<param path="encoding" value="UTF-8"/>
//					<param path="useIDs" value="true"/>
//				</extractor>
//			</am3.saveModel>
			if (extractor.getName().equals(AM3AntToolKit.XMIEXTRACTOR)) {
				// TODO Create a method saveModel(ASMModel modelToSave, String modelPath, Map parameters)
				try {
					ml.save((ASMModel)getProject().getReference(model), path);
				} catch (IOException e) {
					throw new BuildException(e);
				}
			}
			else if (extractor.getName().equals(AM3AntToolKit.XMLEXTRACTOR)) {
				AM3AntToolKit.xmlExtraction((ASMModel)getProject().getReference(model), path, getProject());
			}
//			<am3.saveModel model="" path="">
//				<extractor path="ebnf">
//					<param path="format" value="monModelTCSQuiAEteDejaCharge" />
//					<param path="indentString" value="	"/>
//					<param path="serializeComments" value="false"/>
//				</extractor>
//			</am3.saveModel>
			else if (extractor.getName().equals(AM3AntToolKit.EBNFEXTRACTOR)) {
//				AM3AntToolKit.ebnfExtraction((ASMModel)AM3AntToolKit.getModel(model), path, emfamh, extractor.getParams());
				AM3AntToolKit.ebnfExtraction((ASMModel)getProject().getReference(model), path, extractor.getParams(), getProject());
			}
//			<am3.saveModel model="" path="">
//				<extractor path="atl">
//					<param path="queryPath" value="" />
//				</extractor>
//			</am3.saveModel>
			else if (extractor.getName().equals(AM3AntToolKit.ATLEXTRACTOR)) {
				Object ref = getProject().getReference(model);
				if (ref instanceof ASMModel) {
					URL asmUrl = AM3AntToolKit.getURLFromASMFile(extractor.getParam("queryPath"), getProject().getBaseDir().getAbsolutePath());
					AM3AntToolKit.atlExtractor((ASMModel)ref, path, asmUrl);
				}
			}
		}
	}

	public void setModel(String model) {
		this.model = model;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public void addConfiguredExtractor(Extractor e) {
		extractors.add(e);
	}
	
}
