/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.tasks;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.gmt.am3.tools.ant.toolkit.AM3AntToolKit;
import org.eclipse.m2m.atl.engine.vm.ATLVMPlugin;
import org.osgi.framework.Bundle;

/**
 * 
 * @author Freddy Allilaire
 *
 */
public class UseClasspathTask extends Task {

	private String name;

	public void execute() throws BuildException {
		AM3AntToolKit.attachLogHandler(getProject());
		
		try {
			Path path = new Path(getProject());
			getProject().addReference(name, path);
			
			Bundle bundle = org.eclipse.m2m.atl.dsls.Activator.getDefault().getBundle();
			URL urlPlatform = bundle.getEntry("/");
			URL localFileInPluginURL;
			localFileInPluginURL = FileLocator.toFileURL(urlPlatform);
			String atlEnginePath = localFileInPluginURL.getPath().toString();
			
			path.add(new Path(getProject(), atlEnginePath + "libraries/org.eclipse.gmt.tcs.injector.jar"));
			
			urlPlatform = Platform.getBundle("org.antlr.runtime").getEntry("/");
			localFileInPluginURL = FileLocator.toFileURL(urlPlatform);
			path.add(new Path(getProject(), localFileInPluginURL.getPath().toString()));
			
			urlPlatform = ATLVMPlugin.getDefault().getBundle().getEntry("/");
			localFileInPluginURL = FileLocator.toFileURL(urlPlatform);
			String vmPath = localFileInPluginURL.getPath().toString();
	
			addClassPathIfExists(path, vmPath + "bin/");	//for runtime-workspaces
			addClassPathIfExists(path, vmPath + "vm.jar");	// old packaging
			addClassPathIfExists(path, vmPath);			// new packaging
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void addClassPathIfExists(Path classpath, String path) {
		File file = new File(path);
		if(file.exists()) {
			classpath.add(new Path(getProject(), path));
		}
	}
	
	public void setName(String name) {
		this.name = name;
	}
}
