/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.toolkit;

import java.util.Hashtable;
import java.util.logging.Logger;

import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.BuildListener;
import org.apache.tools.ant.SubBuildListener;
import org.eclipse.m2m.atl.engine.vm.ATLVMPlugin;

/**
 * Listens for build events and cleans up after Ant builds.
 * @author Dennis Wagelaar <dennis.wagelaar@vub.ac.be>
 *
 */
public class AM3BuildListener implements SubBuildListener {

	protected static Logger logger = Logger.getLogger(ATLVMPlugin.LOGGER);

	private String reference;
	private int subBuildCounter = 0;
    
    /**
     * Creates an AM3BuildListener
     * @param reference Reference to remove on buildFinished
     * @see #buildFinished(BuildEvent)
     */
    public AM3BuildListener(String reference) {
        this.reference = reference;
    }
    
    /**
     * Clean up ASMModel references after build finishes.
     * @see BuildListener#buildFinished(BuildEvent)
     */
    public void buildFinished(BuildEvent event) {
        Hashtable refs = event.getProject().getReferences();
        if (refs.containsKey(reference)) {
//            Object refval = refs.get(reference);
//            if (refval instanceof ASMModel) {
//                AtlModelHandler.getHandler((ASMModel)refval).disposeOfModel((ASMModel)refval);
//            } else if (refval instanceof ResourceSet) {
//            	for (Iterator it = ((ResourceSet)refval).getResources().iterator(); it.hasNext();) {
//            		((Resource)it.next()).unload();
//            		it.remove();
//            	}
//            }
            refs.remove(reference);
        } else {
            logger.warning("Reference " + reference + " does not exist");
        }
    }

    public void buildStarted(BuildEvent event) {
    }

    public void messageLogged(BuildEvent event) {
    }

    public void targetFinished(BuildEvent event) {
    }

    public void targetStarted(BuildEvent event) {
    }

    public void taskFinished(BuildEvent event) {
    }

    public void taskStarted(BuildEvent event) {
    }

	public void subBuildFinished(BuildEvent event) {
		// Don't clean up inherited references here, since they may
		// cause NPEs in the super-build!
		if (subBuildCounter > 0) {
			subBuildCounter--;
		} else {
			this.buildFinished(event);
		}
	}

	public void subBuildStarted(BuildEvent event) {
		subBuildCounter++;
	}

}
