/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.toolkit;

import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.BuildListener;

/**
 * Listens for the end of a build, after which the ant task log
 * handler is removed from the external logger object. This prevents
 * class loader leaks, in which a new log handler is registered for
 * every execution of Ant. 
 * @author Dennis Wagelaar <dennis.wagelaar@vub.ac.be>
 */
public class LogHandlerListener implements BuildListener {
	
	public static LogHandlerListener INSTANCE = new LogHandlerListener();
	
	/**
	 * Removed log handler at end of build, before we get unloaded
	 */
	public void buildFinished(BuildEvent event) {
		AM3AntToolKit.logger.removeHandler(AM3AntToolKit.am3LogHandler);
		AM3AntToolKit.logger.fine("Removed AM3 ant log handler");
	}

	public void buildStarted(BuildEvent event) {
	}

	public void messageLogged(BuildEvent event) {
	}

	public void targetFinished(BuildEvent event) {
	}

	public void targetStarted(BuildEvent event) {
	}

	public void taskFinished(BuildEvent event) {
	}

	public void taskStarted(BuildEvent event) {
	}

}
